#pragma once

#include <map>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include "mystd.h"
#include "myexception.h"
/*
#include "value.h"
#include "range.h"
*/

typedef boost::shared_ptr<class Node> NodePtr;
typedef boost::shared_ptr<class Func> FuncPtr;

//! Internal representation of expressions as syntax trees.

static const char *NodeTypeNames[] = { "undef", "mixed", "int", "dbl", "str", "monop", "binop" };

class Node {
 public:
    // WARNING: do not reuse constant names from xax. For some obscure reasons,
    // Node::Types does not live in a different namespace.
    enum Types { UNDEF, MIXED, INT, DBL, STR, MONOP, BINOP };
    virtual Types typ() const { return UNDEF; };
    string typ_strg() const { return NodeTypeNames[typ()]; }; 
    virtual string explicate() const =0;
    virtual void evaluate();
    virtual const Node* as_value() const { cout << "stupid as_value\n"; return this; };
    string represent() const { return as_value()->explicate(); } ;
    virtual bool is_num() const { return false; };
    virtual int as_int() const { throw ProgramError( "invalid call to as_int" ); };
    virtual double as_dbl() const { throw ProgramError( "invalid call to as_dbl" ); };
    virtual string as_str() const { throw ProgramError( "invalid call to as_str" ); };
    virtual void SetArg( NodePtr nod0in )
        { throw ProgramError( "invalid call to SetArg" ); };
    virtual void SetArg( NodePtr nod0in, NodePtr nod1in )
        { throw ProgramError( "invalid call to SetArg" ); };
};

class NodeNum : public Node {
 public:
    bool is_num() const { return true; };
};

class NodeInt : public NodeNum {
 public:
    virtual Types typ() const { return INT; };
    NodeInt( int i_in ) : i(i_in) {};
    int i;
    int as_int() const { return i; };
    double as_dbl() const { return i; };
    string as_str() const { return strg(i); };
    string explicate() const { return as_str(); };
};

class NodeDbl : public NodeNum {
 public:
    virtual Types typ() const { return DBL; };
    NodeDbl( double d_in ) : d(d_in) {};
    double d;
    double as_dbl() const { return d; };
    string as_str() const; // slightly non-trivial implementation
    string explicate() const { return as_str(); };
};

class NodeStr : public Node {
 public:
    virtual Types typ() const { return STR; };
    NodeStr( string s_in ) : s(s_in) {};
    string s;
    string as_str() const { return s; }; 
    string explicate() const { return "\"" + as_str() + "\""; };
};

class NodeOpr : public Node {
 protected:
    FuncPtr f;
 public:
    NodeOpr( const FuncPtr &f_in );
    NodeOpr( const string f_sigle );
    NodePtr evaluated;
    const Node* as_value() const { cout << "smart as_value\n"; return &*evaluated; };
};
    
class NodeMonop : public NodeOpr {
 private:
    NodePtr nod0;
 public:
    NodeMonop( const FuncPtr &f_in ) : NodeOpr( f_in ) {}; 
    NodeMonop( const NodePtr nod0_in, const string f_sigle ) 
        : nod0( nod0_in ), NodeOpr( f_sigle ) {};
    void SetArg( NodePtr nod0in ) { nod0 = nod0in; };
    string explicate() const;
    void evaluate();
};

class NodeBinop : public NodeOpr {
 private:
    NodePtr nod0;
    NodePtr nod1;
 public:
    NodeBinop( const NodePtr nod0_in, const NodePtr nod1_in, const string f_sigle )
        : nod0( nod0_in ), nod1( nod1_in ), NodeOpr( f_sigle ) {};
    void SetArg( NodePtr nod0in, NodePtr nod1in ) { nod0 = nod0in; nod1 = nod1in; };
    string explicate() const;
    void evaluate();
};

class Func {
 public:
    string sigle;
    bool infix;
    virtual NodePtr calculate( NodePtr nod0 ) { throw ProgramError( "unexpected call to calculate(1)" ); } ;
    virtual NodePtr calculate( NodePtr nod0, NodePtr nod1 ) { throw ProgramError( "unexpected call to calculate(2)" ); } ;
};


//! The command registry, to be instantiated once.
class Functions : public map<string, FuncPtr> {
 public:
    //! Register one command.
    void add_one( FuncPtr func, const string sigle_in ){
        if( sigle_in.substr(0,5)=="infix" ){
            func->infix = true;
            func->sigle = sigle_in.substr(6);
        } else {
            func->infix = false;
            func->sigle = sigle_in;
        }
        cout << "registering " << func->sigle << "\n";
        insert( make_pair( func->sigle, func ) ); };
    //! Improve STL's find:
    FuncPtr value_of( const string &key ) {
        map<string,FuncPtr>::iterator pos;
        if ( (pos = find(key)) == end() ) return FuncPtr();
        return pos->second;
    }

};

//! Singleton, to be instantiated in main.
extern Functions theFunctions;

namespace NFunctions {
    void register_functions();
};
