%option noyywrap

%{
using namespace std;
#include <math.h>      // atof
#include <string>      // for func.h
#include "myexception.h"
#include "node.h"
/*
#include "func.h"
*/
#include "xax_yacc.h"

#define YYSTYPE NodePtr
#define YY_DECL int xaxlex(YYSTYPE *xaxlval)
%}

DIG      [0-9]
ID       [a-zA-Z][a-zA-Z0-9\+\-]*
EXP      [eE]"-"?[0-9]+
%%

         /* "#"[.]*  /* comment */

("'"[^']*"'")|("\""[^\"]*"\"") {
        // cout << "found string " << xaxtext << "\n"; 
        string xaxmid = xaxtext;
	*xaxlval = NodePtr( new NodeStr( xaxmid.substr(1,xaxmid.length()-2) ));
	return STRG; }

[ \t\n]+  /* ignore whitespace */

{DIG}+	{
	// printf( "lex: An integer: %s (%d)\n", xaxtext, atoi( xaxtext ) );
        //cout << "going to copy\n";
	*xaxlval = NodePtr( new NodeInt( atoi(xaxtext) ));
        //cout << "copied to " << (**xaxlval).strg() << "\n";
	return NUM; }	

({DIG}+"."{DIG}+{EXP}?)|("."?{DIG}+{EXP}?) {
	// printf( "lex: A float: %s (%g)\n", xaxtext, atof( xaxtext ) );
	*xaxlval = NodePtr( new NodeDbl( atof(xaxtext) ));
	return NUM; }

{DIG}".." { /* will always be interpreted as integer followed by ".."-operator */
        yyless( yyleng-2 ); // push back the ".."
	*xaxlval = NodePtr( new NodeInt( atoi(xaxtext) ));
	return NUM; }

{DIG}"." {
	*xaxlval = NodePtr( new NodeDbl( atof(xaxtext) ));
	return NUM; }

[xykji]|n[ij]|z{DIG}+|p{DIG}+|fqi{DIG}+ {     
        // printf( "lex: The variable %s\n", xaxtext );
//        xaxlval->ptr = (void*) xaxtext; 
        return REF; }

f {     
        // printf( "lex: The variable %s\n", xaxtext );
//        xaxlval->ptr = (void*) xaxtext; 
        return FRF; }

pi {
	// printf( "lex: pi\n" );
	*xaxlval = NodePtr( new NodeDbl( acos(-1.) ));
	return NUM; }

e {
	// printf( "lex: e\n" );
	*xaxlval = NodePtr( new NodeDbl( exp(1.) ));
	return NUM; }

"=="  { return BOP_EQ; }
"!="  { return BOP_NE; }
"<"   { return BOP_LT; }
"<="  { return BOP_LE; }
">"   { return BOP_GT; }
">="  { return BOP_GE; }
"&&"  { return BOP_AND; }
"||"  { return BOP_OR; }
"xor" { return BOP_XOR; }
"mod" { return BOP_MOD; } /* man 3 fmod */
"over" { return BOP_OVER; } /* binomial coefficient: n over k */
"in"  { return BOP_IN; }
".."  { return BOP_RGE; }

{ID}	{
        FuncPtr f;
        f = theFunctions.value_of(xaxtext);
        if ( !f )
            throw BadInput( "identifier \"" + string(xaxtext) +
                            "\" is neither command nor function" );
	*xaxlval = NodePtr( new NodeMonop( f ) );
	return FNCT;
 }

"!"|"-"|"+"|"*"|"/"|"%"|"^"|"="|"("|")"|"["|"]"|","|"?"|":" {           
	// printf( "lex: Special character: %s\n", xaxtext );
	return xaxtext[0]; }

.       {           
	throw BadInput( "lex: invalid special character: " + string(xaxtext) );
	return 0; }

<<EOF>> { return END; }
%%

int xaxparse( void* ); /* provided by xax.y */

/* The user interface: run lexer on lin, return parser results in A */
int user_xaxparse( const string &lin, NodePtr *A )
{
	int err;

	xax_scan_string( lin.c_str() );
	//printf( "/xax_scan\n" );

	err=xaxparse( (void*) A );
	//printf( "/xax_parse\n" );

	xax_delete_buffer( YY_CURRENT_BUFFER );
	//printf( "/xax_delete\n" );

	return err;
}

/* Called by yyparse on error */
int xaxerror( char *s ) 
{
    throw BadInput( s );
    return 0; // does not matter
}
