#include <stdlib.h>
#include <stdio.h>
#include <iostream>

#include <string>
#include <vector>
#include <algorithm>
using namespace std;

#include "olm.h"

int err;

// ** COlo *********************************************************************

int COlo::SaveHeader(FILE *fd)
    // written for file format ASCII-01
{
    fprintf( fd, "fil %s\n", name.c_str() );
    lDoc.push_back("saved as "+name+" on "+
                   mystd::myasctime(mystd::justnow(),0));
    for ( uint i=0; i<lDoc.size(); i++ )
        fprintf(fd, "doc %s\n", lDoc[i].c_str());

    fprintf( fd, "x   %s\n", xco.fstr().c_str() );
    fprintf( fd, "y   %s\n", yco.fstr().c_str() );
    for ( uint i=0; i<ZCo.size(); i++ )
        fprintf(fd, "z%-2u %s\n", i, ZCo[i].fstr().c_str());

    return 0;
}

// ** COld *********************************************************************

COld::COld(CCoord CX, CCoord CY, CGrid VX, vector<double> VY)
{
    vector<double> VZ;
	
    xco = CX;
    yco = CY;

    VS.push_back(CScan(VX, VY, VZ));
}

int COld::SaveBody( FILE *fd, string fmt )
    // written for file format ASCII-01
{
    if( fmt=="a01" ){
       	fprintf( fd, "#j  %u\n", VS.size() );
  	for( uint j=0; j<VS.size(); j++ ){
            fprintf( fd, "scn %u\n", j );
            if( ( err=VS[j].Save( fd, fmt )) ) return err;;
	}
	return 0;
    } else if( fmt=="csv" ){
/*
        for( uint iz=0; iz<ZCo.size(); ++iz ){
            fprintf( fd, "z%d\t", iz );
            for( uint j=0; j<VS.size(); j++ ){
                fprintf( fd, "%g\t\t", VS[j].z[iz] );
            }
            fprintf( fd, "\n" );
        }
*/
        for( uint j=0; j<VS.size(); j++ ){
            for( uint i=0; i<VS[j].size(); ++i ){
                fprintf( fd, "%g\t", VS[j].y[i] );
            }
            fprintf( fd, "\n" );
        } 
	return 0;
    } else if( fmt=="tab" ){
        int n = nPts();
        if ( n==0 ){
            printf( "export to tab only for rectangular tab\n" );
            return 1;
        }
        for( uint i=0; i<n; ++i ){
            for( uint j=0; ; j++ ){
                fprintf( fd, "%g", VS[j].y[i] );
                if( j==VS.size()-1 ) break;
                fprintf( fd, "\t" );
            }
            fprintf( fd, "\n" );
        } 
	return 0;
    } else if( fmt=="lis" ){ // sozusagen außer Betrieb
  	for( uint j=0; j<VS.size(); j++ ){
            if( ( err=VS[j].Save( fd, fmt )) ) return err;;
            if( j<VS.size()-1 )
                fprintf(fd, "\n", j); // ??? // empty line separates scans
            return 0;
        } 
    } else {
        return -1;
    }
}

uint COld::nPts(uint j) const {
    if (!nScan() || j>=nScan()) return 0;
    return VS[j].size();
}
	
uint COld::nPts(void) const {
    if (!nScan()) return 0;
    uint np = VS[0].size();
    for (uint j=0; j<nScan(); ++j)
        if (VS[j].size()!=np) return 0;
    return np;
}
	
// ** COlc *********************************************************************

/*
int COlc::SaveBody(FILE *fd)
    // written for file format ASCII-01
{
    fprintf(fd, "#j  %u\n", VC.size());
    for (uint j=0; j<VC.size(); j++) {
        fprintf(fd, "scn %u\n", j);
        if ((err=VC[j].Save(fd))) return err;;
    }
    return 0;
}
*/

// ** CEle *********************************************************************

void CEle::Z2R(void)
{
    uint k, j, ik;
    vector<int> Agenda;
    for (k=0; k<P()->ZCo.size(); ++k) {
        vector<double> ZK(nScan());
        for (j=0; j<nScan(); ++j) ZK[j] = (*Z(j))[k];
        if (mystd::get_const(&ZK)==0) Agenda.push_back(k);
    }
    for (ik=0; ik<Agenda.size(); ++ik) {
        k = Agenda[ik];
        P()->RPar.push_back(CParam(P()->ZCo[k],(*Z(0))[k]));
    }
    for (ik=Agenda.size()-1; ik!=(uint)-1; --ik) {
        k = Agenda[ik];
        P()->ZCo.erase(P()->ZCo.begin()+k);
        for (j=0; j<nScan(); ++j)
            Z(j)->erase(Z(j)->begin()+k);
    }
}

int CEle::Save( FILE *fd, string fmt ) 
{
    if( fmt=="a01"){
	fprintf(fd, "ASCII-01 JWu\n");

        if     (Typ==CURVE)
            fprintf(fd, "typ C\n");
        else if(Typ==DATA)
            fprintf(fd, "typ D\n");
        else {
            printf("PROGRAM ERROR Save Ele_type\n");
            exit(7);
	}
	
	if ((err=((COlo*) Ptr)->SaveHeader(fd))) return err;

	if        (Typ==DATA) {
            if ((err=((COld*) Ptr)->SaveBody( fd, fmt ))) return err;
//	} else if (Typ==CURVE) {
//            if ((err=((COlc*) Ptr)->SaveBody( fd ))) return err;
	} else 
            exit(7);
        
	fprintf(fd, "eof\n");
        ((COlo*) Ptr)->as_on_disk = true;
	return 0;

    } else if( ( fmt=="csv" || fmt=="tab" ) && Typ==DATA ) {
        if ((err=((COld*) Ptr)->SaveBody( fd, fmt ))) return err;
        return 0;

    } else {
        printf( "CEle::Save> unknown format\n" );
        return -1;
    }
};

int CEle::Load( FILE *fd, string fshort )
{
//    COlc C;
    COld D;
    CScan DS;
//    CCurve CS;
    CCoord co;
    uint iz, nj=0, j;
    string lin, test;


    if (mystd::freadln(fd, &lin)<=0) {
        printf("! file empty or starting with empty line\n");
        goto errex;
    }
    if(lin!=string("ASCII-01 JWu")) {
        printf("! unknown file format %s\n", lin.c_str());
        goto errex;
    }

    if (mystd::freadln(fd, &lin)<=0) {
        printf("! unexpected empty line\n");
        goto errex;
    }
    if (lin.substr(0,4)!=string("typ ")) {
        printf("! missing \"typ\" entry\n");
        goto errex;
    }
    // printf(".. found type line %s\n", lin.c_str());
    if        (lin.substr(4)=="D") {
        Typ = DATA;
        Ptr = &D;
//    } else if (lin.substr(4)=="C") {
//        Typ = CURVE;
//        Ptr = &C;
    } else {
        printf("! invalid type [%s]\n", lin.substr(4).c_str());
        goto errex;
    }

    P()->name = fshort;

    while (1) {
        if (mystd::freadln(fd, &lin)<0) {
            printf("! unexpected end-of-file\n");
            goto errex;
        }
        // printf(".. found line %s\n", lin.c_str());
        if        (lin=="eof") {
            NOlm::OloAdd(P())->P()->as_on_disk = true;
            return 0; // this is the regular exit
        } else if (lin.substr(0,4)=="fil ") {
            if (lin.substr(4)!=fshort) {
                printf("! warning: file %s renamed from %s\n",
                       fshort.c_str(), lin.substr(4).c_str());
            }
        } else if (lin.substr(0,4)=="doc ") {
            P()->lDoc.push_back(lin.substr(4));
        } else if (lin.substr(0,4)=="x   ") {
            P()->xco.load(lin.substr(4));
        } else if (lin.substr(0,4)=="y   ") {
            P()->yco.load(lin.substr(4));
        } else if (lin[0]=='z') {
            if (sscanf(lin.substr(1,3).c_str(), "%u", &iz)!=1) {
                printf("z coordinates must be numbered\n");
                goto errex;
            }
            if (iz!=P()->ZCo.size()) {
                printf("z coordinates must be sorted\n");
                goto errex;
            }
            co.load(lin.substr(4));
            P()->ZCo.push_back(co);
        } else if (lin.substr(0,4)=="#j  ") {
            if (sscanf(lin.substr(4).c_str(), "%u", &nj)!=1) {
                printf("after label #j no no. of scans\n");
                goto errex;
            }
        } else if (lin.substr(0,4)=="scn ") {
            if (sscanf(lin.substr(4).c_str(), "%u", &j)!=1) {
                printf("! number of scan not given\n");
            } else if (j!=nScan()) {
                printf("! reading scan %u, labelled %u\n",
                       nScan(), j);
            }
            if        (Typ == DATA) {
                if (DS.Load(fd, P()->ZCo.size()))
                    goto errex;
                D.VS.push_back(DS);
//            } else if (Typ == CURVE) {
//                if (CS.Load(fd, P()->ZCo.size()))
//                    goto errex;
//                C.VC.push_back(CS);
            } else 
                exit(7);
        } else {
            printf("! Unexpected label [%s]\n", 
                   lin.substr(0,4).c_str());
            break;
        }
    }
 errex:
    return -1;
}

// ** NOlm *********************************************************************

namespace NOlm { // on-line memory (in olm.h predefined as extern)
    // the data:
    vector<class CEle> MOM;

    // the current selection:
    class CList FSel;
    class CList FSelOld;
    bool overwrite;

    // the current scan selection:
    bool FJaskperF;
    CList FJSel;
};

void NOlm::SelNew(void)
{
    FSelOld = FSel;
    FSel = CList();
}

void NOlm::SelPop(void)
{
    FSel = FSelOld;
    FSelOld = CList();
}

int NOlm::OloDup( COlo *olo_in, CEle *ele_out)
// duplicate the online object *olo, though not quite literally, because
// the new object will be packed into an online element. To unpack the
// return value, use ->P().
{
    if        ( COld* olf = dynamic_cast<COld*>(olo_in) ) {
        COld *p = new COld;
        *p = *olf;
        *ele_out = CEle( p );
        return 0;
//    } else if ( COlc* olc = dynamic_cast<COlc*>(olo_in) ) {
//        COlc *p = new COlc;
//        *p = *olc;
//        *ele_out = CEle( p );
//        return 0;
    } else {
        cout << "PROGAMM ERROR: invalid on-line object in OloDup\n";
//        ele_out = 0;
        return -1;
    }
}    

CEle* NOlm::OloAdd(COlo *olo)
{
    CEle e;
    if( OloDup( olo, &e ) ) return 0;
    e.P()->as_on_disk = false;
    return OloPushBack( &e );
}

CEle* NOlm::OloPushBack( CEle *e )
{
    MOM.push_back( *e );
    FSel += CList(MOM.size()-1);
    return &( MOM.back() );
    // cout << "DEBUG now we have " << MOM.size() << " on-line objects\n";
}

void NOlm::OlfDel(void)
{
    for (uint i = FSel.n()-1; i!=(uint)-1; --i) {
        MOM[FSel.V[i]].predelete();
        MOM.erase(MOM.begin()+FSel.V[i]);
    }
    SelPop();
    SelPop();
    SelNew();
    FSel = CList(0,MOM.size()-1);
}

void NOlm::OlfCopy(void)
{
    CEle *e;
    IterateEle eiter;
    while ((e=eiter()))
        OloAdd(e->P());
}

void NOlm::OlfLoop(int(*exec)(int fno, COld *fin, COld *fout))
{
    IterateD fiter;
    if (!overwrite) SelNew();
    COld *fin, ftmp, *fout;
    int fno;
    // cout << "OlfLoop Begin\n";
    while((fin=fiter())) {
        fno = fiter.SelNo();
        cout << "-> operate on file " << fno << "\n";
        if (overwrite) {
            fout = fin;
        } else {
            ftmp = *fin; // copy everything, then modify
            fout = &ftmp;
        }

        if(exec(fno, fin, fout)) return;

        if(!(overwrite)) OloAdd(&ftmp);
    }
    // cout << "OlfLoop End\n";
}

int NOlm::FJSelAsk (string quest)
{
    IterateEle iter;
    CEle *e;
    if (FSel.n()<=0) return 1;

    uint nJ = MOM[FSel.V[0]].nScan();
    FJaskperF = 0;
    while ((e=iter())) {
        if (e->nScan()!=nJ) {
            cout << ".. files have different number of scans\n";
            FJaskperF = 1;
            break;
        }
    }
	
    if (FJaskperF) return 0;

    if (nJ==1) {
        FJSel = CList("0");
        return 0;
    }

    // printf("DEBUG FJSelAsk nJ=%d\n", nJ);
    FJSel.Ask(quest.c_str(), FJSel, CLimits(0, nJ-1));

    return 0;
}

void NOlm::FJSelGet (uint fno, class CList *JSel)
{
    if (FJaskperF) {
        // printf("DEBUG going to ask per file (fno=%u)\n", fno);
        if (fno>=MOM.size()) {
            printf("SEVERE PROG ERR in FJSelGet invalid fno=%u,"
                   " size=%u\n", fno, MOM.size());
            *JSel = CList();
            return;
        }
        uint nJ = MOM[fno].nScan();
        string quest = "Enter spectra for file " + strg(fno);
        // printf("DEBUG going to call JSel->Ask\n");
        JSel->Ask(quest.c_str(), FJSel, CLimits(0, nJ-1));
        // printf("DEBUG back from JSel->Ask\n");
    } else {
        *JSel = FJSel;
    }
}

void NOlm::FJSelGetNoask (uint fno, class CList *JSel)
{
    if (FJaskperF) {
        // printf("DEBUG FJS Get Noask\n");
        *JSel = CList(0, MOM[fno].nScan()-1);
        // printf("DEBUG FJS Get Noask ok\n");
    } else {
        *JSel = FJSel;
    }
}
