//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* mem.h: online memory                                                   *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.mesen-und-deuten.de/frida                                   *//
//**************************************************************************//

namespace NOlm {
    // the data:
    extern vector<POlo> MOM; // the mother of all on-line data

    // the current selection:
    extern CList FSel;
    extern CList FSelOld;
    extern CList FSelNew;
    extern bool overwrite;

    // modify selection or/and data:
    void SelPush( CList& _FSel );
    void SelPop();
    void SelAssert();
    void OloAdd( POlo olo, int k=-1 );
    void OloAdd( POld old, int k=-1 );
    void OloAdd( POlc olc, int k=-1 );
    void OlfDel();
    void OlfExchange();
    void OlfCopy();

    // spectrum selection:
    void JSelAsk( string quest, string *jSel, CList *JSel );

    class IterateBase { // iterator over FSel (virtual base)
    protected:
        uint iV;
        CList Sel;
    public:
        IterateBase() { Sel = FSel; reset(); };
        void reset() { iV = 0; };
        void choose_all() { Sel = CList(0,MOM.size()-1); };
        uint k() const { return Sel.V[iV-1]; }
        uint iteration() const { return iV-1; }
        uint size() const { return Sel.V.size(); }
    };
    class IterateO : public IterateBase { // all types
    public:
        POlo operator()() {
            if (iV>=Sel.size()) return POlo();
            return MOM[Sel.V[iV++]];
        };
    };
    class IterateD : public IterateBase { // must be data file
    public:
        POld operator()() {
            POld e;
            while ( iV<Sel.size() && !(e = P2D( MOM[Sel.V[iV++]] )) ) {};
            return e;
        };
    };
    class IterateC : public IterateBase { // must be curve file
    public:
        POlc operator()() {
            POlc e;
            while ( iV<Sel.size() && !(e = P2C( MOM[Sel.V[iV++]] )) ) {};
            return e;
        };
    };
};
