//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* var.cpp: variable type and number                                      *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.mesen-und-deuten.de/frida                                   *//
//**************************************************************************//

#include "mystd.h"
#include "var.h"

//***************************************************************************//
//* class CVariable                                                         *//
//***************************************************************************//

//! Variables may be unnumbered (x,y,i,j,k,f,...) or numbered (z0,p0,...).

CVariable::CVariable( string s )
{
    num = -1;
    int isub;
    char c;
    // WARNING TO PROGRAMMERS:
    // As long as this constructor is called by the parser and not
    // by the lexical analyzer, string s may contain more than just the
    // parameter name. Everything after the parameter name must be ignored.
    // In other words: the stupid .substr(0,2) are NOT superfluous.
    int nsub = sscanf( s.c_str(), "%c%d", &c, &isub );
    if ( s.substr(0,2)=="cq" && sscanf(s.substr(2).c_str(), "%d", &isub)==1 ) {
        typ = _CQ;
        num = isub;
    } else if ( nsub==2 && c=='p' ) {
        typ = _CP;
        num = isub;
    } else if ( s.substr(0,2)=="z+" ) {
        typ = _Z; // new z value
    } else if ( nsub==2 && c=='z' ) {
        typ = _Z;  // existing z value
        num = isub;
    } else if ( nsub==2 && c=='r' ) {
        typ = _R;  // existing r value
        num = isub;
    } else if ( s.substr(0,2)=="nj" ) {
        typ = _NJ;
    } else if ( s.substr(0,2)=="ni" ) {
        typ = _NI;
    } else if ( c=='x' ) {
        typ = _X;
    } else if ( c=='y' ) {
        typ = _Y;
    } else if ( s.substr(0,2)=="dy" ) {
        typ = _DY;
    } else if ( c=='c' ) {
        typ = _C;
    } else if ( c=='k' ) {
        typ = _K;
    } else if ( c=='j' ) {
        typ = _J;
    } else if ( c=='i' ) {
        typ = _I;
    } else {
        throw "invalid variable at beginning of string '" + s + "'";
        // typ = _NOREF;
    }
}


//! Return associated error variable.

CVariable CVariable::errvar() const
{
    switch (typ) {
    case _Y:
        return CVariable( _DY );  break;
    default:
        return CVariable();
    }
}


//! Convert to text.

string CVariable::var_info() const
{
    string ret;
    switch (typ) {
    case _X:  ret = "x";  break;
    case _Y:  ret = "y";  break;
    case _DY: ret = "dy";  break;
    case _C:  ret = "c";  break;
    case _Z:  ret = "z";  break;
    case _R:  ret = "r";  break;
    case _CP: ret = "p";  break;
    case _CQ: ret = "cq"; break;
    case _K:  ret = "k";  break;
    case _J:  ret = "j";  break;
    case _I:  ret = "i";  break;
    case _NJ: ret = "nj"; break;
    case _NI: ret = "ni"; break;
    default:
        throw string( "BUG: no info available for unexpected variable typ" );
    }
    if ( numbered() )
        ret += strg(num);
    return ret;
}
